import logging

import wx


def filename_repr(filenames):
    if filenames:
        if isinstance(filenames, str):
            filenames = [filenames]

#        if 'wxMSW' in wx.PlatformInfo:
            # does windows require separate handling for to backslash?
            # on os-x, if the path contains a backslash, it is inserted as '\\'
#            conv = []
#            for name in filenames:
#                try:
#                    conv.append('r%r' % str(name))
#                except:
#                    conv.append(repr(name))
#
#            if len(filenames) == 1:
#                string = filenames[0]
#            else:
#                string = '[' + ', '.join(filenames) + ']'
#
#        else:
        try:
            filenames = tuple(map(str, filenames))
        except BaseException:
            pass

        logging.debug(filenames)

        if len(filenames) == 1:
            string = repr(filenames[0])
        else:
            string = repr(filenames)

        return string
    else:
        return ''


class FilenameDropTarget(wx.FileDropTarget):
    """
    File drop target: http://wiki.wxpython.org/DragAndDrop

    (!) apparently the FileDropTarget can only be used once; when I saved the
    instance as an Editor attribute and SetDropTarget it to a second editor
    this caused a segmentation fault.

    """

    def __init__(self, text_target):
        wx.FileDropTarget.__init__(self)
        self.text_target = text_target

    def OnDropFiles(self, x, y, filenames):
        msg = "DROP! %r" % filenames
        logging.info(msg)
        if len(filenames) == 1:
            filenames = "'%s'" % (filenames[0])
        self.text_target.ReplaceSelection(str(filenames))


class TextDropTarget(wx.TextDropTarget):
    def __init__(self, text_target):
        wx.TextDropTarget.__init__(self)
        self.text_target = text_target

    def OnDropText(self, x, y, text):
        msg = "DROP! %r" % text
        logging.info(msg)
        self.text_target.ReplaceSelection(text)


class StringDropTarget(wx.DropTarget):
    """
    DropTarget for multiple data types

    based on:
    http://www.wiki.wxpython.org/DragAndDrop#wxDataObjectComposite

    """

    def __init__(self, target):
        wx.DropTarget.__init__(self)
        self.target = target

        self.do = wx.DataObjectComposite()  # the dataobject that gets filled with the appropriate data
        self.filedo = wx.FileDataObject()
        self.textdo = wx.TextDataObject()
#        self.bmpdo = wx.BitmapDataObject()
        self.do.Add(self.filedo)
        self.do.Add(self.textdo)
#        self.do.Add(self.bmpdo)
        self.SetDataObject(self.do)

    def OnData(self, x, y, d):
        if self.GetData():
            df = self.do.GetReceivedFormat().GetType()
#            data = self.GetData()

            if df in [wx.DF_UNICODETEXT, wx.DF_TEXT]:
                string = self.textdo.GetText()
            elif df == wx.DF_FILENAME:
                filenames = self.filedo.GetFilenames()
                string = filename_repr(filenames)

            msg = "OnData! %r" % string
            logging.info(msg)
            self.target.ReplaceSelection(string)
        return d


def set_for_strings(window):
    drop_target = StringDropTarget(window)
    window.SetDropTarget(drop_target)



# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
from os.path import join
from warnings import catch_warnings, filterwarnings

import mne
from eelbrain import gui, load
from eelbrain.testing import gui_test, TempDir
from eelbrain._wxgui import ID


@gui_test
def test_select_components():
    "Test Select-Epochs GUI Document"
    tempdir = TempDir()
    path = join(tempdir, 'test-ica.fif')

    data_path = mne.datasets.testing.data_path()
    raw_path = join(data_path, 'MEG', 'sample', 'sample_audvis_trunc_raw.fif')
    raw = mne.io.Raw(raw_path, preload=True)
    ds = load.mne.events(raw, stim_channel='STI 014')
    ds['epochs'] = load.mne.mne_epochs(ds, tmax=0.1)
    ica = mne.preprocessing.ICA(0.95, max_iter=1)
    with catch_warnings():
        filterwarnings('ignore', 'FastICA did not converge')
        ica.fit(raw)
    ica.save(path)

    frame = gui.select_components(path, ds)
    frame.model.toggle(1)
    frame.OnSave(None)
    ica = mne.preprocessing.read_ica(path)
    assert ica.exclude == [1]

    frame.OnUndo(None)
    frame.OnSave(None)
    ica = mne.preprocessing.read_ica(path)
    assert ica.exclude == []

    # plotting
    for i in [ID.BASELINE_NONE, ID.BASELINE_GLOABL_MEAN, ID.BASELINE_CUSTOM]:
        frame.butterfly_baseline = i
        frame.OnPlotGrandAverage(None)

    frame.Close()



import cProfile
import os
import pstats

import mne
import numpy as np
import eelbrain
from eelbrain import *

fname = 'profile_of_adjacency.profile'

mne.set_log_level('warning')
configure(n_workers=False)

ds = datasets.get_mne_sample(-0.1, 0.2, src='ico', sub="modality == 'A'")

code = '''
res = testnd.TTestIndependent('src', 'side', 'L', 'R', ds=ds, samples=5, pmin=0.5,
                       tstart=0.05, mintime=0.02, minsource=10)
'''

cProfile.run(code, fname)

p = pstats.Stats(fname)
p.sort_stats('cumulative')
p.print_stats(20)



# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
import sys

import numpy as np
import pytest

from eelbrain import datasets, plot, _info, NDVar
from eelbrain.testing import hide_plots, requires_mne_testing_data


@hide_plots
@requires_mne_testing_data
@pytest.mark.mayavi
def test_plot_brain():
    """Test plot.brain plots"""
    if sys.platform.startswith('win'):
        pytest.xfail("Hangs on Appveyor")
    stc = datasets.get_mne_stc(True, 'oct-4')

    # size
    b = plot.brain.brain(stc.source, hemi='rh', w=400, h=300, mask=False)
    assert b.screenshot().shape in ((300, 400, 3), (600, 800, 3))
    if sys.platform == 'linux':
        pytest.xfail("Brain.set_size() on Linux/Travis")
    b.set_size(200, 150)
    assert b.screenshot().shape in ((150, 200, 3), (300, 400, 3))
    b.close()
    # both hemispheres [broken upstream?]
    # b = plot.brain.brain(stc.source, w=600, h=300, mask=False)
    # assert b.screenshot().shape in ((300, 600, 3), (600, 1200, 3))
    # b.set_size(400, 150)
    # assert b.screenshot().shape in ((150, 400, 3), (300, 800, 3))
    # b.close()

    # standard plot
    y = stc.sub(time=0.090)
    brain = plot.brain.brain(y, mask=False, hemi='lh')
    # labels (int)
    labels = y.astype(int)
    brain.add_ndvar_annotation(labels)
    brain.add_ndvar_annotation(labels, 'jet')
    brain.close()

    # plot shortcuts
    p = plot.brain.dspm(stc, mask=False)
    cb = p.plot_colorbar()
    cb.close()
    p.close()

    p = plot.brain.dspm(stc, hemi='lh', mask=False)
    cb = p.plot_colorbar()
    cb.close()
    p.close()

    p = plot.brain.cluster(stc, hemi='rh', views='parietal', mask=False)
    cb = p.plot_colorbar()
    cb.close()
    p.close()

    image = plot.brain.bin_table(stc, tstart=0.05, tstop=0.07, tstep=0.01, surf='white', mask=False)
    print(repr(image))
    print(image)

    # plot p-map
    pmap = NDVar(np.random.uniform(0, 1, stc.shape), stc.dims, _info.for_p_map(stc.info))
    p = plot.brain.p_map(pmap, stc, mask=False)
    cb = p.plot_colorbar()
    cb.close()
    p.close()

    # mask
    stcm = stc.mask(stc < 11)
    p = plot.brain.brain(stcm, mask=False)
    p.close()


@hide_plots
@requires_mne_testing_data
@pytest.mark.mayavi
def test_sequence_plotter():
    if sys.platform.startswith('win'):
        pytest.xfail("Hangs on Appveyor")
    stc = datasets.get_mne_stc(True)
    stc_mask = stc > 5

    # time dimension
    y = stc.sub(time=(0.000, 0.200)).bin(0.050)
    y_mask = stc_mask.sub(time=(0.000, 0.200)).bin(0.050, func='max')
    sp = plot.brain.SequencePlotter()
    sp.set_brain_args(surf='white', mask=False)  # the source space required for mask is not in the test dataset
    sp.add_ndvar(y, vmax=10)
    sp.add_ndvar(y_mask)
    sp.plot_table(view='lateral')
    # test internals
    assert sp._get_frame_labels(True) == ['0 - 50 ms', '50 - 100', '100 - 150', '150 - 200']

    # separate NDVars
    sp = plot.brain.SequencePlotter()
    sp.set_brain_args(mask=False)
    sp.add_ndvar(stc.mean(time=(0.150, 0.250)), label='normal')
    sp.add_ndvar(stc.mean(time=(0.150, 0.250)), cmap='jet', label='jet')
    sp.add_ndvar(stc_mask.max(time=(0.150, 0.250)), label='mask')
    sp.plot_table(view='lateral', orientation='vertical')
    # test internals
    assert sp._get_frame_labels(True) == ['normal', 'jet', 'mask']



# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
"""
.. _exa-compare-topographies:

Compare topographies
====================

.. currentmodule:: eelbrain

This example shows how to compare EEG topographies, based on the method described by McCarthy & Wood [1]_. We have also applied this method to source localized data, to distinguish true localization differences from apparent differences due to the sensitivity of souce estimate extent to amplitude [2]_.
"""
# sphinx_gallery_thumbnail_number = 4
from eelbrain import *

###############################################################################
# Simulated data
# --------------
# Generate a simulated dataset (as in the :ref:`exa-cluster-based-mu` example)
dss = []
for subject in range(10):
    # generate data for one subject
    ds = datasets.simulate_erp(seed=subject)
    # average across trials to get condition means
    ds_agg = ds.aggregate('predictability')
    # add the subject name as variable
    ds_agg[:, 'subject'] = f'S{subject:02}'
    dss.append(ds_agg)

ds = combine(dss)
# make subject a random factor (to treat it as random effect for ANOVA)
ds['subject'].random = True
# Re-reference the EEG data (i.e., subtract the mean of the two mastoid channels):
ds['eeg'] -= ds['eeg'].mean(sensor=['M1', 'M2'])
ds.head()

###############################################################################
# The simulated data in the two conditions:

p = plot.TopoArray('eeg', 'predictability', data=ds, columns=1, axh=2, axw=10, t=[0.120, 0.200, 0.280], head_radius=0.35)

###############################################################################
# Test between conditions
# -------------------------------------
#
# Test whether the 120 ms topography differs between the two cloze conditions. The dataset already includes one row per cell (i.e., per cloze condition and subject). Consequently, we can just index the topography at the desired time point:

topography = ds['eeg'].sub(time=0.120)
# normalize the data in accordance with McCarth & Wood (1985)
topography = normalize_in_cells(topography, 'sensor', 'predictability', ds)
# "melt" the topography NDVar to turn the sensor dimension into a Factor
ds_topography = table.melt_ndvar(topography, 'sensor', ds=ds)
# Note EEG is a single column, and the last column indicates the sensor 
ds_topography.head()

###############################################################################
# ANOVA to test whether the effect of `predictability` differs between sensors:

test.ANOVA('eeg', 'predictability * sensor * subject', data=ds_topography)

###############################################################################
# The non-significant interaction suggests that the effect of `predictability` does not differ between sensors, i.e., the topographies do not differ, which is consistent with being generated by the same underlying neural sources.
#
#
# Test two time points
# ---------------------
# Since we're not interested in condition here, we first average across conditions, i.e., with the goal of having one row per subject:

ds_average = ds.aggregate('subject', drop_bad=True)
print(ds_average)

###############################################################################
# In order to compare two time points, we need to construct a new dataset with time point as :class:`Factor`:

dss = []
for time in [0.120, 0.280]:
    ds_time = ds_average['subject',]  # A new dataset with the 'subject' variable only
    ds_time['eeg'] = ds_average['eeg'].sub(time=time)
    ds_time[:, 'time'] = f'{time*1000:.0f} ms'
    dss.append(ds_time)
ds_times = combine(dss)
ds_times.summary()

###############################################################################
# Then, normalize the data in accordance with McCarth & Wood (1985)

topography = normalize_in_cells('eeg', 'sensor', 'time', data=ds_times)
# "melt" the topography NDVar to turn the sensor dimension into a Factor
ds_topography = table.melt_ndvar(topography, 'sensor', ds=ds_times)
# Note EEG is a single column, and the last column indicates the sensor 
ds_topography.head()

###############################################################################
# Plot the topographies before and after normalization:

p = plot.Topomap('eeg', 'time', data=ds_times, columns=2, title="Original", head_radius=0.35)
p = plot.Topomap(topography, 'time', data=ds_times, columns=2, title="Normalized", head_radius=0.35)

###############################################################################
# Compare the topographies with the ANOVA -- test whether the effect of time differs between sensors:
test.ANOVA('eeg', 'time * sensor * subject', data=ds_topography)

###############################################################################
# The significant interaction suggests that the effect of time differs between sensors.
# This means the two topographic patterns are different, suggesting that they were generated by a different constellation of underlying sources.
# Visualize the difference:

res = testnd.TTestRelated(topography, 'time', match='subject', data=ds_times)
p = plot.Topomap(res, columns=3, title="Normalized topography differences", head_radius=0.35)

###############################################################################
# References
# ----------
# .. [1] McCarthy, G., & Wood, C. C. (1985). Scalp Distributions of Event-Related Potentials—An Ambiguity Associated with Analysis of Variance Models. Electroencephalography and Clinical Neurophysiology, 61, S226–S227. `10.1016/0013-4694(85)90858-2 <https://doi.org/10.1016/0013-4694(85)90858-2>`_
# .. [2] Brodbeck, C., Bhattasali, S., Cruz Heredia, A. A., Resnik, P., Simon, J. Z., & Lau, E. (2022). Parallel processing in speech perception with local and global representations of linguistic context. eLife, 11, e72056. `10.7554/eLife.72056 <https://doi.org/10.7554/eLife.72056>`_
#



# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
"""
.. _exa-lm:

Multiple regression
===================

Multiple regression designs apply to at least two scenarios.

 - Analyzing a single subject's data when trials are associated with one or multiple continuous variables.
 - Analyzing group data when subjects are associated with one or multiple individual difference variables.

Here the analysis is illustrated for a simulated dataset from a single subject. Group analysis works analogously, except that each case in the datatset would represent a different subject rather than a different trial. 

For such group analysis, it is necessary to reduce each subject's data to a single case first because multiple regression assumes a fixed effects model. 
Such designs are described under :ref:`exa-two-stage`.

Simulated data
--------------
"""
# sphinx_gallery_thumbnail_number = 2
from eelbrain import *


data = datasets.simulate_erp(snr=0.5)
data.head()

###############################################################################
# The data represents 80 trials from a simulated word reading paradigm, where each word is associated with a word length (`n_chars`) and predictability (`cloze`).

p = plot.Scatter('cloze', 'n_chars', data=data, h=3)

###############################################################################
# Plot the average of each level on `n_chars` to illustrate the linear increase of the response around 130 ms with `n_char`.

p = plot.TopoButterfly('eeg', 'n_chars', data=data, t=.130, axh=2, w=6)

###############################################################################
# Multiple regression
# -------------------
# Fit a multiple regression model. Estimate $p$-values using a cluster-based permuatation test, using a cluster-forming threshold of uncorrected $p$=.05 and 10,000 permutations (the default).

lm = testnd.LM('eeg', 'n_chars + cloze', data=data, tstart=0.050, tstop=0.500, pmin=0.05)

###############################################################################
# A quick plot suggests an early effect related to `n_chars` around 130 ms, and a later effect of `cloze` around 400 ms:

p = plot.TopoButterfly(lm, t=0.130, axh=2, w=6)

###############################################################################
# Detailed plots of the two effects show that the cluster is quite variable due to noise in the data.

p = plot.TopoArray(lm.masked_parameter_map('n_chars'), t=[0.110, 0.130, 0.150], title='N Chars')
p = plot.TopoArray(lm.masked_parameter_map('cloze'), t=[0.380, 0.400, 0.420], title='Cloze')

###############################################################################
# To access some of the test results we need to know the index of different effects:

lm.effects

###############################################################################
# Create a plot that shows the spatial cluster extent across time.
EFFECTS = [
    ('n_chars', (0.110, 0.150)),
    ('cloze', (0.380, 0.420)),
]
    
t_maps = []
for effect, time in EFFECTS:
    # t-maps are retrieved by effect name
    t = lm.t(effect).mean(time=time)
    # p-maps are stored in a list, so we need to know the index of te effect
    index = lm.effects.index(effect)
    # We are interested in the maximal spatial cluster extent, i.e., any sensor that is part of the cluster at any time 
    p = lm.p[index].min(time=time)
    # Create a masked average t map
    t_av = t.mask(p > 0.05)
    t_maps.append(t_av)

p = plot.Topomap(t_maps, columns=2)



# skip test: data unavailable
from eelbrain import *

from mouse import e


res = e.load_test('surprise', 0.3, 0.5, 0.05, epoch='target', make=True, raw='fastica1-40', inv='fixed-3-dSPM', mask='frontotemporal-lh')
plot.brain.butterfly(res)



from logging import getLogger
import packaging.version
import select
import sys
from threading import Thread
from time import sleep
import webbrowser

import wx
from wx.adv import TaskBarIcon, TBI_DOCK

from .._utils import IS_OSX, IS_WINDOWS
from ..plot._base import CONFIG
from .about import AboutFrame
from .frame import FOCUS_UI_UPDATE_FUNC_NAMES, EelbrainFrame
from .utils import Icon
from . import ID


APP = None  # hold the App instance
JUMPSTART_TIME = 250  # ms


def wildcard(filetypes):
    if filetypes:
        return '|'.join(map('|'.join, filetypes))
    else:
        return ""


class App(wx.App):
    _pt_thread = None
    about_frame = None
    _result = None
    _bash_ui_from_mainloop = None
    _ipython = None
    using_prompt_toolkit = False

    def OnInit(self):
        self.SetAppName("Eelbrain")
        self.SetAppDisplayName("Eelbrain")

        # register in IPython
        if CONFIG['prompt_toolkit'] and 'IPython' in sys.modules and packaging.version.parse(sys.modules['IPython'].__version__) >= packaging.version.parse('5'):
            import IPython.core.error

            if CONFIG['prompt_toolkit'] == 'eelbrain':
                import IPython.terminal.pt_inputhooks
                import IPython.core.pylabtools

                IPython.terminal.pt_inputhooks.register('eelbrain', self.pt_inputhook)
                IPython.core.pylabtools.backend2gui.clear()  # prevent pylab from initializing event-loop
            shell = IPython.get_ipython()
            if shell is not None:
                self._pt_thread = self._pt_thread_win if IS_WINDOWS else self._pt_thread_linux
                try:
                    shell.enable_gui(CONFIG['prompt_toolkit'])
                except IPython.core.error.UsageError:
                    print(f"Prompt-toolkit does not seem to be supported by the current IPython shell ({shell.__class__.__name__}); The Eelbrain GUI needs to block Terminal input to work. Use eelbrain.gui.run() to start GUI interaction.")
                else:
                    self.using_prompt_toolkit = True
                    self._ipython = shell
                    getLogger('Eelbrain').debug("Initialized prompt_toolkit with %s", CONFIG['prompt_toolkit'])

        self.SetExitOnFrameDelete(not self.using_prompt_toolkit)

        if IS_OSX:
            self.dock_icon = DockIcon(self)
            self.menubar = self.CreateMenu(self)
            # list windows in Window menu
            self.window_menu_window_items = []
            self.Bind(wx.EVT_MENU_OPEN, self.OnMenuOpened)
        else:
            self.dock_icon = None
            self.menu_bar = None
            self.window_menu_window_items = None

        return True

    def CreateMenu(self, t):
        """Create Menubar

        Parameters
        ----------
        t : App | EelbrainFrame
            Object to which the menu will be attached; on macOS ``self``, on
            other systems the specific :class:`EelbrainFrame`.
        """
        menu_bar = wx.MenuBar()

        # File Menu
        m = file_menu = wx.Menu()
        m.Append(wx.ID_OPEN, '&Open... \tCtrl+O')
        m.AppendSeparator()
        m.Append(wx.ID_CLOSE, '&Close Window \tCtrl+W')
        m.Append(wx.ID_SAVE, "Save \tCtrl+S")
        m.Append(wx.ID_SAVEAS, "Save As... \tCtrl+Shift+S")
        menu_bar.Append(file_menu, "File")

        # Edit Menu
        m = edit_menu = wx.Menu()
        m.Append(ID.UNDO, '&Undo \tCtrl+Z')
        m.Append(ID.REDO, '&Redo \tCtrl+Shift+Z')
        m.AppendSeparator()
        m.Append(wx.ID_CUT, 'Cut \tCtrl+X')
        m.Append(wx.ID_COPY, 'Copy \tCtrl+C')
        m.Append(ID.COPY_AS_PNG, 'Copy as PNG \tCtrl+Shift+C')
        m.Append(wx.ID_PASTE, 'Paste \tCtrl+V')
        m.AppendSeparator()
        m.Append(wx.ID_CLEAR, 'Cle&ar')
        menu_bar.Append(edit_menu, "Edit")

        # Tools Menu
        # updated by the active GUI
        if IS_OSX or hasattr(t, 'MakeToolsMenu'):
            tools_menu = wx.Menu()
            if not IS_OSX:
                t.MakeToolsMenu(tools_menu)
            menu_bar.Append(tools_menu, "Tools")

        # View Menu
        m = view_menu = wx.Menu()
        m.Append(ID.SET_VLIM, "Set Axis Limits... \tCtrl+l", "Change the current figure's axis limits")
        m.Append(ID.LINK_TIME_AXES, "Link Time Axes", "Synchronize the time displayed on figures")
        m.Append(ID.SET_MARKED_CHANNELS, "Mark Channels...", "Mark specific channels in plots")
        m.Append(ID.DRAW_CROSSHAIRS, "Draw &Crosshairs", "Draw crosshairs under the cursor", kind=wx.ITEM_CHECK)
        m.AppendSeparator()
        m.Append(ID.SET_LAYOUT, "&Set Layout... \tCtrl+Shift+l", "Change the page layout")
        menu_bar.Append(view_menu, "View")

        # Go Menu
        m = go_menu = wx.Menu()
        m.Append(wx.ID_FORWARD, '&Forward \tCtrl+]', 'Go One Page Forward')
        m.Append(wx.ID_BACKWARD, '&Back \tCtrl+[', 'Go One Page Back')
        m.Append(ID.TIME, '&Time... \tCtrl+t', 'Go to time...')
        if not self.using_prompt_toolkit:
            m.AppendSeparator()
            m.Append(ID.YIELD_TO_TERMINAL, '&Yield to Terminal \tAlt+Ctrl+Q')
        menu_bar.Append(go_menu, "Go")

        # Window Menu
        m = window_menu = wx.Menu()
        m.Append(ID.WINDOW_MINIMIZE, '&Minimize \tCtrl+M')
        m.Append(ID.WINDOW_ZOOM, '&Zoom')
        m.Append(ID.SET_TITLE, '&Set Title')
        m.AppendSeparator()
        m.Append(ID.WINDOW_TILE, '&Tile')
        m.AppendSeparator()
        menu_bar.Append(window_menu, "Window")

        # Help Menu
        m = help_menu = wx.Menu()
        m.Append(ID.HELP_EELBRAIN, 'Eelbrain Help')
        m.Append(ID.HELP_PYTHON, "Python Help")
        m.AppendSeparator()
        m.Append(wx.ID_ABOUT, '&About Eelbrain')
        menu_bar.Append(help_menu, self.GetMacHelpMenuTitleName() if IS_OSX else 'Help')

        # Menu Bar
        wx.MenuBar.MacSetCommonMenuBar(menu_bar)

        # Bind Menu Commands
        t.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)
        t.Bind(wx.EVT_MENU, t.OnOpen, id=wx.ID_OPEN)
        t.Bind(wx.EVT_MENU, t.OnClear, id=wx.ID_CLEAR)
        t.Bind(wx.EVT_MENU, t.OnWindowClose, id=wx.ID_CLOSE)
        t.Bind(wx.EVT_MENU, t.OnCopy, id=wx.ID_COPY)
        t.Bind(wx.EVT_MENU, self.OnCopyAsPNG, id=ID.COPY_AS_PNG)
        t.Bind(wx.EVT_MENU, self.OnCut, id=wx.ID_CUT)
        t.Bind(wx.EVT_MENU, t.OnDrawCrosshairs, id=ID.DRAW_CROSSHAIRS)
        t.Bind(wx.EVT_MENU, self.OnOnlineHelp, id=ID.HELP_EELBRAIN)
        t.Bind(wx.EVT_MENU, self.OnOnlineHelp, id=ID.HELP_PYTHON)
        t.Bind(wx.EVT_MENU, self.OnPaste, id=wx.ID_PASTE)
        t.Bind(wx.EVT_MENU, t.OnRedo, id=ID.REDO)
        t.Bind(wx.EVT_MENU, t.OnSave, id=wx.ID_SAVE)
        t.Bind(wx.EVT_MENU, t.OnSaveAs, id=wx.ID_SAVEAS)
        t.Bind(wx.EVT_MENU, t.OnSetLayout, id=ID.SET_LAYOUT)
        t.Bind(wx.EVT_MENU, t.OnSetMarkedChannels, id=ID.SET_MARKED_CHANNELS)
        t.Bind(wx.EVT_MENU, t.OnSetVLim, id=ID.SET_VLIM)
        t.Bind(wx.EVT_MENU, t.OnSetTime, id=ID.TIME)
        t.Bind(wx.EVT_MENU, self.OnLinkTimeAxes, id=ID.LINK_TIME_AXES)
        t.Bind(wx.EVT_MENU, t.OnUndo, id=ID.UNDO)
        t.Bind(wx.EVT_MENU, t.OnWindowIconize, id=ID.WINDOW_MINIMIZE)
        t.Bind(wx.EVT_MENU, self.OnWindowTile, id=ID.WINDOW_TILE)
        t.Bind(wx.EVT_MENU, t.OnWindowZoom, id=ID.WINDOW_ZOOM)
        t.Bind(wx.EVT_MENU, t.OnSetWindowTitle, id=ID.SET_TITLE)
        t.Bind(wx.EVT_MENU, self.OnQuit, id=wx.ID_EXIT)
        t.Bind(wx.EVT_MENU, self.OnYieldToTerminal, id=ID.YIELD_TO_TERMINAL)

        # UI-update concerning frames
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIBackward, id=wx.ID_BACKWARD)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIClear, id=wx.ID_CLEAR)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIClose, id=wx.ID_CLOSE)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIDown, id=wx.ID_DOWN)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIDrawCrosshairs, id=ID.DRAW_CROSSHAIRS)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIForward, id=wx.ID_FORWARD)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIOpen, id=wx.ID_OPEN)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIRedo, id=ID.REDO)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUISave, id=wx.ID_SAVE)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUISaveAs, id=wx.ID_SAVEAS)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUISetLayout, id=ID.SET_LAYOUT)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUISetMarkedChannels, id=ID.SET_MARKED_CHANNELS)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUISetVLim, id=ID.SET_VLIM)
        t.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUILinkTimeAxes, id=ID.LINK_TIME_AXES)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUISetTime, id=ID.TIME)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUITools, id=ID.TOOLS)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIUndo, id=ID.UNDO)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIUp, id=wx.ID_UP)

        # UI-update concerning focus
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIFocus, id=wx.ID_COPY)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIFocus, id=ID.COPY_AS_PNG)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIFocus, id=wx.ID_CUT)
        t.Bind(wx.EVT_UPDATE_UI, t.OnUpdateUIFocus, id=wx.ID_PASTE)

        return menu_bar

    def _pt_thread_win(self, context):
        # On Windows, select.poll() is not available
        while context._input_is_ready is None or not context.input_is_ready():
            sleep(0.020)
        wx.CallAfter(self.ExitMainLoop, True)

    def _pt_thread_linux(self, context):
        # getLogger('Eelbrain').debug("PTK: poll")
        poll = select.poll()
        poll.register(context.fileno(), select.POLLIN)
        poll.poll(-1)
        wx.CallAfter(self.ExitMainLoop, True)

    def pt_inputhook(self, context):
        """prompt_toolkit inputhook"""
        # prompt_toolkit.eventloop.inputhook.InputHookContext
        Thread(target=self._pt_thread, args=(context,)).start()
        self.MainLoop()

    def jumpstart(self):
        wx.CallLater(JUMPSTART_TIME, self.ExitMainLoop)
        self.MainLoop()

    def _get_active_frame(self):
        win = wx.Window.FindFocus()
        win_parent = wx.GetTopLevelParent(win)
        if win_parent:
            return win_parent
        for w in wx.GetTopLevelWindows():
            if hasattr(w, 'IsActive') and w.IsActive():
                return w
        return wx.GetActiveWindow()

    def _get_parent_gui(self):
        frame = self._get_active_frame()
        if frame is None:
            return
        while True:
            if hasattr(frame, 'MakeToolsMenu'):
                return frame
            elif frame.Parent is not None:
                frame = frame.Parent
            else:
                return

    def _bash_ui(self, func, *args):
        "Launch a modal dialog based on terminal input"
        # Create fake frame to prevent dialog from sticking
        if not self.GetTopWindow():
            self.SetTopWindow(wx.Frame(None))
        # Run dialog
        self._bash_ui_from_mainloop = self.using_prompt_toolkit or self.IsMainLoopRunning()
        if self._bash_ui_from_mainloop:
            return func(*args)
        else:
            wx.CallAfter(func, *args)
            print("Please switch to the Python Application to provide input.")
            self.MainLoop()
            return self._result

    def _bash_ui_finalize(self, result):
        if self._bash_ui_from_mainloop:
            self.Yield()
            return result
        else:
            self._result = result
            self.ExitMainLoop()

    def ask_for_dir(self, title="Select Folder", message="Please Pick a Folder",
                    must_exist=True):
        return self._bash_ui(self._ask_for_dir, title, message, must_exist)

    def _ask_for_dir(self, title, message, must_exist):
        style = wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
        if must_exist:
            style = style | wx.DD_DIR_MUST_EXIST

        dialog = wx.DirDialog(None, message, name=title,
                              style=style)
        dialog.SetTitle(title)
        if dialog.ShowModal() == wx.ID_OK:
            result = dialog.GetPath()
        else:
            result = False
        dialog.Destroy()
        return self._bash_ui_finalize(result)

    def ask_for_file(self, title, message, filetypes, directory, mult):
        return self._bash_ui(self._ask_for_file, title, message, filetypes, directory, mult)

    def _ask_for_file(self, title, message, filetypes, directory, mult):
        """Return path(s) or False.

        Parameters
        ----------
        ...
        directory : str
            Path to initial directory.

        Returns
        -------
        result : list | str | None
            Paths(s) or False.
        """
        style = wx.FD_OPEN
        if mult:
            style = style | wx.FD_MULTIPLE
        dialog = wx.FileDialog(None, message, directory, wildcard=wildcard(filetypes), style=style)
        dialog.SetTitle(title)
        if dialog.ShowModal() == wx.ID_OK:
            if mult:
                result = dialog.GetPaths()
            else:
                result = dialog.GetPath()
        else:
            result = False
        dialog.Destroy()
        return self._bash_ui_finalize(result)

    def ask_for_string(self, title, message, default='', parent=None):
        return self._bash_ui(self._ask_for_string, title, message, default, parent)

    def _ask_for_string(self, title, message, default, parent):
        dialog = wx.TextEntryDialog(parent, message, title, default)
        if dialog.ShowModal() == wx.ID_OK:
            result = dialog.GetValue()
        else:
            result = False
        dialog.Destroy()
        return self._bash_ui_finalize(result)

    def ask_saveas(self, title, message, filetypes, defaultDir, defaultFile):
        return self._bash_ui(self._ask_saveas, title, message, filetypes, defaultDir, defaultFile)

    def _ask_saveas(self, title, message, filetypes, defaultDir, defaultFile):
        # setup file-dialog
        dialog = wx.FileDialog(None, message, wildcard=wildcard(filetypes), style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        dialog.SetTitle(title)
        if defaultDir:
            dialog.SetDirectory(defaultDir)
        if defaultFile:
            dialog.SetFilename(defaultFile)
        # get result
        if dialog.ShowModal() == wx.ID_OK:
            result = dialog.GetPath()
        else:
            result = False
        dialog.Destroy()
        return self._bash_ui_finalize(result)

    def message_box(self, message, caption, style, parent=None):
        return self._bash_ui(self._message_box, message, caption, style, parent)

    def _message_box(self, message, caption, style, parent):
        dialog = wx.MessageDialog(parent, message, caption, style)
        result = dialog.ShowModal()
        dialog.Destroy()
        return self._bash_ui_finalize(result)

    def ExitMainLoop(self, event_with_pt=True):
        if event_with_pt or not self.using_prompt_toolkit:
            # with prompt-toolkit, this leads to hanging when terminating the
            # interpreter
            wx.App.ExitMainLoop(self)

    def Attach(self, obj, desc, default_name, parent):
        if self._ipython is None:
            self.message_box("Attach Unavailable", "The attach command requires running from within IPython 5 or later", wx.ICON_ERROR | wx.OK, parent)
            return
        name = self.ask_for_string("Attach", f"Variable name for {desc} in terminal:", default_name, parent)
        if name:
            self._ipython.user_global_ns[name] = obj

    def OnAbout(self, event):
        if not self.about_frame:
            self.about_frame = AboutFrame(None)
        self.about_frame.Show()
        self.about_frame.Raise()

    def OnClear(self, event):
        frame = self._get_active_frame()
        frame.OnClear(event)

    def OnCopy(self, event):
        win = wx.Window.FindFocus()
        if hasattr(win, 'CanCopy'):
            return win.Copy()
        win = self._get_active_frame()
        if hasattr(win, 'CanCopy'):
            return win.Copy()
        getLogger('Eelbrain').debug("App.OnCopy() call but neither focus nor frame have CanCopy()")
        event.Skip()

    def OnCopyAsPNG(self, event):
        wx.Window.FindFocus().CopyAsPNG()

    def OnCut(self, event):
        win = wx.Window.FindFocus()
        win.Cut()

    def OnDrawCrosshairs(self, event):
        frame = self._get_active_frame()
        frame.OnDrawCrosshairs(event)

    def OnLinkTimeAxes(self, event):
        from ..plot._base import TimeSlicer
        from .._data_obj import UTS

        figures = []
        for window in wx.GetTopLevelWindows():
            eelfigure = getattr(window, '_eelfigure', None)
            if eelfigure and isinstance(eelfigure, TimeSlicer) and isinstance(eelfigure._time_dim, UTS):
                figures.append(eelfigure)
        if len(figures) >= 2:
            f0 = figures[0]
            for figure in figures[1:]:
                f0.link_time_axis(figure)

    def OnMenuOpened(self, event):
        "Update the window-specific Tools menu"
        menu = event.GetMenu()
        if menu.GetTitle() == 'Tools':
            for item in menu.GetMenuItems():
                menu.Remove(item)
            frame = self._get_parent_gui()
            if frame:
                frame.MakeToolsMenu(menu)

    def OnOnlineHelp(self, event):
        "Called from the Help menu to open external resources"
        event_id = event.GetId()
        if event_id == ID.HELP_EELBRAIN:
            webbrowser.open("https://pythonhosted.org/eelbrain/")
        elif event_id == ID.HELP_PYTHON:
            webbrowser.open("http://docs.python.org/2.7/")
        else:
            raise RuntimeError("Invalid help ID")

    def OnOpen(self, event):
        frame = self._get_active_frame()
        frame.OnOpen(event)

    def OnPaste(self, event):
        win = wx.Window.FindFocus()
        win.Paste()

    def OnQuit(self, event):
        getLogger('Eelbrain').debug("App.OnQuit()")
        for win in wx.GetTopLevelWindows():
            if not win.Close():
                return
        getLogger('Eelbrain').debug("App.ExitMainLoop()")
        self.ExitMainLoop()

    def OnRedo(self, event):
        frame = self._get_active_frame()
        frame.OnRedo(event)

    def OnSave(self, event):
        frame = self._get_active_frame()
        frame.OnSave(event)

    def OnSaveAs(self, event):
        frame = self._get_active_frame()
        frame.OnSaveAs(event)

    def OnSetVLim(self, event):
        frame = self._get_active_frame()
        frame.OnSetVLim(event)

    def OnSetLayout(self, event):
        frame = self._get_active_frame()
        frame.OnSetLayout(event)

    def OnSetMarkedChannels(self, event):
        frame = self._get_active_frame()
        frame.OnSetMarkedChannels(event)

    def OnSetTime(self, event):
        frame = self._get_active_frame()
        frame.OnSetTime(event)

    def OnSetWindowTitle(self, event):
        frame = self._get_active_frame()
        frame.OnSetWindowTitle(event)

    def OnUndo(self, event):
        frame = self._get_active_frame()
        frame.OnUndo(event)

    def OnUpdateUIBackward(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIBackward'):
            frame.OnUpdateUIBackward(event)
        else:
            event.Enable(False)

    def OnUpdateUIClear(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIClear'):
            frame.OnUpdateUIClear(event)
        else:
            event.Enable(False)

    def OnUpdateUIClose(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIClose'):
            frame.OnUpdateUIClose(event)
        else:
            event.Enable(False)

    def OnUpdateUIDown(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIDown'):
            frame.OnUpdateUIDown(event)
        else:
            event.Enable(False)

    def OnUpdateUIDrawCrosshairs(self, event):
        frame = self._get_active_frame()
        if isinstance(frame, EelbrainFrame):
            frame.OnUpdateUIDrawCrosshairs(event)
        else:
            event.Enable(False)
            event.Check(False)

    def OnUpdateUIForward(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIForward'):
            frame.OnUpdateUIForward(event)
        else:
            event.Enable(False)

    def OnUpdateUIFocus(self, event):
        func_name = FOCUS_UI_UPDATE_FUNC_NAMES[event.GetId()]
        win = wx.Window.FindFocus()
        func = getattr(win, func_name, None)
        if func is None:
            win = self._get_active_frame()
            func = getattr(win, func_name, None)
            if func is None:
                event.Enable(False)
                return
        event.Enable(func())

    def OnUpdateUILinkTimeAxes(self, event):
        n = 0
        for window in wx.GetTopLevelWindows():
            eelfigure = getattr(window, '_eelfigure', None)
            if eelfigure:
                n += hasattr(eelfigure, 'link_time_axis')
                if n >= 2:
                    event.Enable(True)
                    return
        event.Enable(False)

    def OnUpdateUIOpen(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIOpen'):
            frame.OnUpdateUIOpen(event)
        else:
            event.Enable(False)

    def OnUpdateUIRedo(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIRedo'):
            frame.OnUpdateUIRedo(event)
        else:
            event.Enable(False)

    def OnUpdateUISave(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISave'):
            frame.OnUpdateUISave(event)
        else:
            event.Enable(False)

    def OnUpdateUISaveAs(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISaveAs'):
            frame.OnUpdateUISaveAs(event)
        else:
            event.Enable(False)

    def OnUpdateUISetLayout(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISetLayout'):
            frame.OnUpdateUISetLayout(event)
        else:
            event.Enable(False)

    def OnUpdateUISetMarkedChannels(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISetMarkedChannels'):
            frame.OnUpdateUISetMarkedChannels(event)
        else:
            event.Enable(False)

    def OnUpdateUISetVLim(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISetVLim'):
            frame.OnUpdateUISetVLim(event)
        else:
            event.Enable(False)

    def OnUpdateUISetTime(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISetTime'):
            frame.OnUpdateUISetTime(event)
        else:
            event.Enable(False)

    def OnUpdateUISetWindowTitle(self, event):
        frame = self._get_active_frame()
        event.Enable(getattr(frame, '_allow_user_set_title', False))

    def OnUpdateUITools(self, event):
        event.Enable(bool(self._get_parent_gui))

    def OnUpdateUIUndo(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIUndo'):
            frame.OnUpdateUIUndo(event)
        else:
            event.Enable(False)

    def OnUpdateUIUp(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIUp'):
            frame.OnUpdateUIUp(event)
        else:
            event.Enable(False)

    def OnWindowClose(self, event):
        frame = self._get_active_frame()
        if frame:
            frame.Close()

    def OnWindowIconize(self, event):
        frame = self._get_active_frame()
        if frame:
            frame.Iconize()

    def OnWindowRaise(self, event):
        id_ = event.GetId()
        window = wx.FindWindowById(id_)
        window.Raise()

    def OnWindowTile(self, event):
        frames = sorted(wx.GetTopLevelWindows(), key=lambda x: x.Position[0])
        dx, dy = wx.DisplaySize()
        x = 0
        y = 0
        y_next = 0
        for frame in frames:
            sx, sy = frame.Size
            if x and x + sx > dx:
                if y_next > dy:
                    return
                x = 0
                y = y_next
            frame.Position = (x, y)
            y_next = max(y_next, y + sy)
            x += sx

    def OnWindowZoom(self, event):
        frame = self._get_active_frame()
        if frame:
            frame.Maximize()

    def OnYieldToTerminal(self, event):
        self.ExitMainLoop()


def get_app(jumpstart=False):
    global APP
    if APP is None:
        try:
            APP = App()
        except SystemExit as exc:
            if exc.code.startswith("This program needs access to the screen"):
                raise SystemExit(
                    f"{exc.code} \n\n"
                    f"If you are using an iPython terminal: make sure you are "
                    f"running a framework build by launching IPython with:\n\n"
                    f"    $ eelbrain\n\n"
                    f"If you are using a Jupyter notebook, prefix the notebook with\n\n"
                    f"    %matplotlib inline\n\n"
                    f"and restart the kernel.")
            else:
                raise

        if jumpstart and IS_OSX:
            # Give wx a chance to initialize the GUI backend
            APP.OnAbout(None)
            wx.CallLater(JUMPSTART_TIME, APP.about_frame.Close)
            wx.CallLater(JUMPSTART_TIME, APP.ExitMainLoop)
            APP.MainLoop()

    return APP


def needs_jumpstart():
    return APP is None and IS_OSX


def run(block=False):
    """Hand over command to the GUI (quit the GUI to return to the terminal)

    Parameters
    ----------
    block : bool
        Block the Terminal even if the GUI is capable of being run in parallel.
        Control returns to the Terminal when the user quits the GUI application.
        This is also useful to prevent plots from closing at the end of a
        script.
    """
    app = get_app()
    if app.using_prompt_toolkit:
        if block:
            app.MainLoop()
    else:
        if not app.IsMainLoopRunning():
            print("Starting GUI. Quit the Python application to return to the "
                  "shell...")
            app.MainLoop()


class DockIcon(TaskBarIcon):
    # http://stackoverflow.com/a/38249390/166700
    def __init__(self, app):
        TaskBarIcon.__init__(self, iconType=TBI_DOCK)
        self.app = app

        # Set the image
        self.SetIcon(Icon('eelbrain256', True), "Eelbrain")
        self.imgidx = 1

    def CreatePopupMenu(self):
        if not self.app.using_prompt_toolkit:
            menu = wx.Menu()
            menu.Append(ID.YIELD_TO_TERMINAL, '&Yield to Terminal')
            return menu



# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
import os

from eelbrain import *
import mne


data_dir = mne.datasets.sample.data_path()
PATH = 'test-ica.fif'

data = mne.io.read_raw_fif(data_dir / 'MEG' / 'sample' / 'sample_audvis_filt-0-40_raw.fif', preload=True)
data.set_eeg_reference()
data.pick_types(eeg=True)
# data.pick_types('mag')
data.filter(1, 40)
if os.path.exists(PATH):
    # ica = mne.preprocessing.read_ica(PATH)
# else:
    ica = mne.preprocessing.ICA()
    # ica = mne.preprocessing.ICA(0.95)
    # ica = mne.preprocessing.ICA(0.99)
    ica.fit(data)
    ica.save(PATH, overwrite=True)
g = gui.select_components(PATH, data)



"""
.. _exa-intro:
.. currentmodule:: eelbrain

Introduction
============

Data are represented with three primary data-objects:

* :class:`Factor` for categorial variables
* :class:`Var` for scalar variables
* :class:`NDVar` for multidimensional data (e.g. a variable measured at
  different time points)

Multiple variables belonging to the same dataset can be grouped in a
:class:`Dataset` object.

.. contents:: Contents
   :local:


Factor
------

A :class:`Factor` is a container for one-dimensional, categorial data: Each
case is described by a string label. The most obvious way to initialize a
:class:`Factor` is a list of strings:

"""
# sphinx_gallery_thumbnail_number = 3
from eelbrain import *

a = Factor(['a', 'a', 'a', 'a', 'b', 'b', 'b', 'b'], name='A')
a

###############################################################################
# Since Factor initialization simply iterates over the given data, the
# same Factor could be initialized with:

a = Factor('aaaabbbb', name='A')
a

###############################################################################
# There are other shortcuts to initialize factors  (see also
# the :class:`Factor` class documentation):

a = Factor(['a', 'b', 'c'], repeat=4, name='A')
a

###############################################################################
# Indexing works like for arrays:

a[0]

###############################################################################
a[0:6]

###############################################################################
# All values present in a :class:`Factor` are accessible in its
# :attr:`Factor.cells` attribute:

a.cells

###############################################################################
# Based on the Factor's cell values, boolean indexes can be generated:

a == 'a'

###############################################################################
a.isany('a', 'b')

###############################################################################
a.isnot('a', 'b')

###############################################################################
# Interaction effects can be constructed from multiple factors with the ``%``
# operator:

b = Factor(['d', 'e'], repeat=2, tile=3, name='B')
b

###############################################################################
i = a % b
i

###############################################################################
# Interaction effects are in many ways interchangeable with factors in places
# where a categorial model is required:

i.cells

###############################################################################
i == ('a', 'd')

###############################################################################
# Var
# ---
#
# The :class:`Var` class is a container for one-dimensional
# :py:class:`numpy.ndarray`:

y = Var([1, 2, 3, 4, 5, 6])
y

###############################################################################
# Indexing works as for factors

y[5]

###############################################################################
y[2:]

###############################################################################
# Many array operations can be performed on the object directly

y + 1

###############################################################################
# For any more complex operations the corresponding :py:class:`numpy.ndarray`
# can be retrieved in the :attr:`Var.x` attribute:

y.x

###############################################################################
# .. Note::
#     The :attr:`Var.x` attribute is not intended to be replaced; rather, a new
#     :class:`Var` object should be created for a new array.
#
#
# NDVar
# -----
#
# :class:`NDVar` objects are containers for multidimensional data, and manage the
# description of the dimensions along with the data. :class:`NDVar` objects are
# usually constructed automatically by an importer function (see
# :ref:`reference-io`), for example by importing data from MNE-Python through
# :mod:`load.mne`.
#
# Here we use data from a simulated EEG experiment as example:

data = datasets.simulate_erp(snr=0.5)
eeg = data['eeg']
eeg

###############################################################################
# This representation shows that ``eeg`` contains 80 trials of data (cases),
# with 140 time points and 35 EEG sensors.
#
# The object provides access to the underlying array...

eeg.x

###############################################################################
# ... and dimension descriptions:

eeg.sensor

###############################################################################
eeg.time

###############################################################################
# Eelbrain functions take advantage of the dimensions descriptions (such as
# sensor locations), for example for plotting:

p = plot.TopoButterfly(eeg, t=0.130)

###############################################################################
# :class:`NDVar` offer functionality similar to :class:`numpy.ndarray`, but
# take into account the properties of the dimensions. For example, through the
# :meth:`NDVar.sub` method, indexing can be done using meaningful descriptions,
# such as indexing a time slice in seconds ...

eeg_130 = eeg.sub(time=0.130)
p = plot.Topomap(eeg_130)
eeg_130

###############################################################################
# ... or extracting data from a specific sensor:

eeg_fz = eeg.sub(sensor='Fz')
p = plot.UTSStat(eeg_fz)
eeg_fz

###############################################################################
# Other methods allow aggregating data, for example an RMS over sensor ...

eeg_rms = eeg.rms('sensor')
plot.UTSStat(eeg_rms)
eeg_rms

###############################################################################
# ... or a mean in a time window:

eeg_average = eeg.mean(time=(0.100, 0.150))
p = plot.Topomap(eeg_average)

###############################################################################
# Dataset
# -------
#
# A :class:`Dataset` is a container for multiple variables
# (:class:`Factor`, :class:`Var` and :class:`NDVar`) that describe the same
# cases. It can be thought of as a data table with columns corresponding to 
# different variables and rows to different cases.
# Consider the dataset containing the simulated EEG data used above:

data

###############################################################################
# Because this can be more output than needed, the
# :meth:`Dataset.head` method only shows the first couple of rows:

data.head()

###############################################################################
# This dataset containes severeal univariate columns: ``cloze``, ``predictability``, and ``n_chars``.
# The last line also indicates that the dataset contains an :class:`NDVar` called ``eeg``.
# The :class:`NDVar` is not displayed as column because it contains many values per row. 
# In the :class:`NDVar`, the :class:`Case` dimension corresponds to the row in the dataset
# (which here corresponds to simulated trial number):

data['eeg']

###############################################################################
# The type and value range of each entry in the :class:`Dataset` can be shown using the :meth:`Dataset.summary` method:

data.summary()

###############################################################################
# An even shorter summary can be generated by the string representation:

repr(data)

###############################################################################
# Here, ``80 cases`` indicates that the Dataset contains 80 rows. The
# subsequent dictionary-like representation shows the keys and the types of the
# corresponding values (``F``: :class:`Factor`, ``V``: :class:`Var`, ``Vnd``: :class:`NDVar`).
#
# Datasets can be indexed with columnn names, ...

data['cloze']

###############################################################################
# ... row numbers, ...

data[2:5]

###############################################################################
# ... or both, in wich case row comes before column:

data[2:5, 'n_chars']

###############################################################################
# Array-based indexing also allows indexing based on the Dataset's variables:

data['n_chars'] == 3

###############################################################################
data[data['n_chars'] == 3]

###############################################################################
# :meth:`Dataset.eval` allows evaluatuing code strings in the namespace
# defined by the dataset, which means that dataset variables can be invoked
# with just their name:

data.eval("predictability == 'high'")

###############################################################################
# Many dataset methods allow using code strings as shortcuts for expressions
# involving dataset variables, for example indexing:

data.sub("predictability == 'high'").head()

###############################################################################
# Columns in the :class:`Dataset` can be used to define models, for statistics,
# aggregating and plotting.
# Any string specified as argument in those functions will be evaluated in the
# dataset, thuse, because we can use:

data.eval("eeg.sub(sensor='Cz')")

###############################################################################
# ... we can quickly plot the time course of a sensor by condition:

p = plot.UTSStat("eeg.sub(sensor='Cz')", "predictability", data=data)

###############################################################################
p = plot.UTSStat("eeg.sub(sensor='Fz')", "n_chars", data=data, colors='viridis')

###############################################################################
# Or calculate a difference wave:

data_average = data.aggregate('predictability')
data_average

###############################################################################
difference = data_average[1, 'eeg'] - data_average[0, 'eeg']
p = plot.TopoArray(difference, t=[None, None, 0.400])

###############################################################################
# For examples of how to construct datasets from scratch see :ref:`exa-dataset`.



# skip test: data unavailable
from eelbrain.pipeline import MneExperiment, RawFilter, RawICA, LabelVar, PrimaryEpoch, SecondaryEpoch, TTestOneSample, TTestRelated, ANOVA


class ImageNet(MneExperiment):

    preload = True

    ignore_entities = {
        'ignore_subjects': ('02', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25', '26', '27', '28', '29', '30', 'emptyroom'),
        'ignore_sessions': ('ImageNet02', 'ImageNet03', 'ImageNet04', 'MRI'),
        'ignore_runs': ('02'),
    }

    raw = {
        '1-40': RawFilter('raw', 1, 40),
        'ica': RawICA('1-40', 'ImageNet', n_components=0.99),
    }

    variables = {
        'position': LabelVar('trigger', {1: 'begin', 2: 'end', (3, 4): 'middle'}),
        'event': LabelVar('trigger', {(1, 2): 'unused', 3: 'resp', 4: 'stim_on'}),
    }

    epochs = {
        'used': PrimaryEpoch('ImageNet', "position == 'middle'", samplingrate=200),
        'resp': SecondaryEpoch('used', "event == 'resp'"),
        'stim_on': SecondaryEpoch('used', "event == 'stim_on'"),
        'cov': SecondaryEpoch('used', tmax=0),
    }

    tests = {
        '=0': TTestOneSample(),
        'connection': TTestRelated('event', 'stim_on', 'resp'),
        'anova': ANOVA('event * subject'),
    }


root = '~/Data/ds005810'
e = ImageNet(root)
